//---------------------------------------------------------------------------


#pragma hdrstop
#include <windows.h>
#include <lmerr.h>
#include "Od_Functions.h"




//---------------------------------------------------------------------------

#pragma package(smart_init)
void __stdcall GetErrorText(DWORD dwLastError, char *ErrText)
{
    if(!dwLastError) return;
    HMODULE hModule = NULL; // default to system source
    LPSTR MessageBuffer;
    DWORD dwBufferLength;

    DWORD dwFormatFlags = FORMAT_MESSAGE_ALLOCATE_BUFFER |
        FORMAT_MESSAGE_IGNORE_INSERTS |
        FORMAT_MESSAGE_FROM_SYSTEM ;

    //
    // If dwLastError is in the network range,
    //  load the message source.
    //

    if(dwLastError >= NERR_BASE && dwLastError <= MAX_NERR) {
        hModule = LoadLibraryEx(
            TEXT("netmsg.dll"),
            NULL,
            LOAD_LIBRARY_AS_DATAFILE
            );

        if(hModule != NULL)
            dwFormatFlags |= FORMAT_MESSAGE_FROM_HMODULE;
    }

    //
    // Call FormatMessage() to allow for message
    //  text to be acquired from the system
    //  or from the supplied module handle.
    //
    dwBufferLength = FormatMessageA(
        dwFormatFlags,
        hModule, // module to get message from (NULL == system)
        dwLastError,
        MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // default language
        (LPSTR) &MessageBuffer,
        0,
        NULL
        );
    if(dwBufferLength)
    {
        //
        // Output message string.
        //
        //****************************************************************
        //MessageBox(NULL,MessageBuffer,"SystemError",MB_OK|MB_ICONERROR);
        if(ErrText)
        {
        	strcpy(ErrText,MessageBuffer);
        }
        //
        // Free the buffer allocated by the system.
        //
        LocalFree(MessageBuffer);
    }

    //
    // If we loaded a message source, unload it.
    //
    if(hModule != NULL)
        FreeLibrary(hModule);
}
//----------------------------------------------------------------------------

void __stdcall DisplayErrorText(DWORD dwLastError)
{
	char ErrText[500];
    GetErrorText(dwLastError, ErrText);
    MessageBox(NULL,ErrText,"SystemError",MB_OK|MB_ICONERROR);
}
//----------------------------------------------------------------------------
void __stdcall ShowLastError()
{
    DisplayErrorText(GetLastError());
}
//----------------------------------------------------------------------------
bool __stdcall WriteToFileStr(const char *FileName, const char *Str, bool ShowErrorOnFail)
{
    HANDLE hFile;

    hFile = CreateFile(FileName,       //   
        FILE_ALL_ACCESS,
        NULL,
        NULL,
        OPEN_ALWAYS,
        FILE_ATTRIBUTE_NORMAL,
        NULL);
    if(hFile == INVALID_HANDLE_VALUE){ //     
        if(ShowErrorOnFail) ShowLastError();               //    
        return false;                  //  false
    }

    if(SetFilePointer(hFile,           //   
        NULL,
        NULL,
        FILE_END)==INVALID_SET_FILE_POINTER){

        if(ShowErrorOnFail) ShowLastError();
        return false;
    }


    DWORD BytesWriten;                 //  
    if(!WriteFile(hFile,
        Str,
        strlen(Str),
        &BytesWriten,
        NULL)){

        if(ShowErrorOnFail) ShowLastError();
        return false;
    }

    char *Buf="\n";
    if(!WriteFile(hFile,               //  
        Buf,
        1,
        &BytesWriten,
        NULL)){

        if(ShowErrorOnFail) ShowLastError();
        return false;
    }

    if(!CloseHandle(hFile)){
        if(ShowErrorOnFail) ShowLastError();
        return false;
    }

    return true;
}
//----------------------------------------------------------------------------
bool __stdcall GetVersionOfFile(const char *FileName, int *VerInfo, bool ShowErrorOnFail)
{
    if(!IsFile(FileName)){ //   
        if(ShowErrorOnFail) ShowLastError();
        return false; //   - 
    }
    char* InternalFileName;
    InternalFileName = (char*)malloc(strlen(FileName)+1);
    strcpy(InternalFileName,FileName);
    DWORD FSize = GetFileVersionInfoSize(InternalFileName,NULL); //     
    if(FSize==0){ //  0  
        if(ShowErrorOnFail) ShowLastError();
        return false;
    }
    LPVOID pBlock = (char*)malloc(FSize); //     
    GetFileVersionInfo(InternalFileName,NULL,FSize,pBlock); //     
    LPVOID MS;
    UINT LS;
    try{
        VerQueryValue(pBlock,"\\",&MS,&LS); //    
    }
    catch(...){
        if(ShowErrorOnFail) ShowLastError();
        return false; //     
    }
    VS_FIXEDFILEINFO FixedFileInfo; //      
    memmove(&FixedFileInfo, MS, LS); //    

    DWORD FileVersionMS = FixedFileInfo.dwFileVersionMS;
    DWORD FileVersionLS = FixedFileInfo.dwFileVersionLS;

    VerInfo[0] = HIWORD(FileVersionMS); //  
    VerInfo[1] = LOWORD(FileVersionMS); //     
    VerInfo[2] = HIWORD(FileVersionLS);
    VerInfo[3] = LOWORD(FileVersionLS);

    return true; //  
}
//----------------------------------------------------------------------------
bool __stdcall IsFile(const char* FileName, bool ShowErrorOnFail)
{
    HANDLE File;
    WIN32_FIND_DATA FD;
    File = FindFirstFile(FileName, &FD); //  
    if(File==INVALID_HANDLE_VALUE || FD.dwFileAttributes==FILE_ATTRIBUTE_DIRECTORY){
        if(ShowErrorOnFail) ShowLastError();
        return false; //     ,  
    }
    if(!FindClose(File)){ //   
        if(ShowErrorOnFail) ShowLastError();
        return false;
    }
    return true; //  
}
//----------------------------------------------------------------------------
bool __stdcall IsDir(const char *DirName, bool ShowErrorOnFail){
    HANDLE File;
    WIN32_FIND_DATA FD;
    File = FindFirstFile(DirName, &FD); //  
    if(File==INVALID_HANDLE_VALUE || FD.dwFileAttributes!=FILE_ATTRIBUTE_DIRECTORY){
        if(ShowErrorOnFail) ShowLastError();
        return false; //     ,  
    }
    if(!FindClose(File)){ //   
        if(ShowErrorOnFail) ShowLastError();
        return false;
    }
    return true; //  
}
//----------------------------------------------------------------------------
void __stdcall PressKey(WORD Key)
{
    keybd_event(Key,0x45,KEYEVENTF_EXTENDEDKEY | 0,0);
    keybd_event(Key,0x45,KEYEVENTF_EXTENDEDKEY | KEYEVENTF_KEYUP,0);
}
//----------------------------------------------------------------------------
bool __stdcall DeleteDir(const char *Path, bool ShowErrorOnFail)
{
    char *IPath;                            //   
    IPath = (char*)malloc(lstrlen(Path)+2); // ,    
    strcpy(IPath,Path);                     //   ..
    IPath[lstrlen(IPath)+1]='\0';           //  1 '\0'

    SHFILEOPSTRUCT strFileOp;               //  
    strFileOp.hwnd = NULL;
    strFileOp.wFunc = FO_DELETE;
    strFileOp.pFrom = IPath;
    strFileOp.pTo = NULL;

    if(!ShowErrorOnFail) strFileOp.fFlags = FOF_NOCONFIRMATION|FOF_NOERRORUI;
    else strFileOp.fFlags = FOF_NOCONFIRMATION;

    if(!SHFileOperation(&strFileOp)) return false;

    return true;

}
//----------------------------------------------------------------------------
bool __stdcall CopyDir(const char *From, const char *To, bool ShowErrorOnFail)
{
    char *IFrom;                            //   
    IFrom = (char*)malloc(lstrlen(From)+2); // ,    
    strcpy(IFrom,From);                     //   ..
    IFrom[lstrlen(IFrom)+1]='\0';           //  1 '\0'

    char *ITo;                              //   
    ITo = (char*)malloc(lstrlen(To)+2);     // ,    
    strcpy(ITo,To);                         //   ..
    ITo[lstrlen(ITo)+1]='\0';               //  1 '\0'

    if(!IsDir(To)){                         //   
        SECURITY_ATTRIBUTES sa;             //,  
        sa.nLength = sizeof(SECURITY_ATTRIBUTES);
        sa.lpSecurityDescriptor = NULL;
        sa.bInheritHandle = false;
        CreateDirectory(To,&sa);
    }

    SHFILEOPSTRUCT strFileOp;               //  
    strFileOp.hwnd = NULL;
    strFileOp.wFunc = FO_COPY;
    strFileOp.pFrom = IFrom;
    strFileOp.pTo = ITo;

    if(!ShowErrorOnFail) strFileOp.fFlags = FOF_NOCONFIRMATION|FOF_NOERRORUI;
    else strFileOp.fFlags = FOF_NOCONFIRMATION;

    if(!SHFileOperation(&strFileOp)) return false;

    return true;
}
//----------------------------------------------------------------------------
bool __stdcall Run(char* Program, bool ShowErrorOnFail){
	STARTUPINFO si;
    PROCESS_INFORMATION pi;

    ZeroMemory( &si, sizeof(si) );
    si.cb = sizeof(si);
    ZeroMemory( &pi, sizeof(pi) );

    //   
    if( !CreateProcess( NULL,   //    (.  )
        Program,                //  
        NULL,                   // Process handle not inheritable.
        NULL,                   // Thread handle not inheritable.
        FALSE,                  // Set handle inheritance to FALSE.
        0,                      // No creation flags.
        NULL,                   // Use parent's environment block.
        NULL,                   // Use parent's starting directory.
        &si,                    // Pointer to STARTUPINFO structure.
        &pi )                   // Pointer to PROCESS_INFORMATION structure.

    )
    {
        if(ShowErrorOnFail) ShowLastError();
        return false;
    }
    else{
        return true;
    }
    // End the child process.
}
//----------------------------------------------------------------------------

